#=====================================================================================================================================
#Copyright
#=====================================================================================================================================

#Copyright (C) 2014 Alexander Blaessle, Patrick Mueller and the Friedrich Miescher Laboratory of the Max Planck Society
#This software is distributed under the terms of the GNU General Public License.

#This file is part of PyFRAP.

#PyFRAP is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

#===========================================================================================================================================================================
#Module Description
#===========================================================================================================================================================================

"""Statistics module for PyFRAP toolbox, mainly used to evaluate goodness of fit, but also providing functions
to assess overall measurement statistics.
"""

#===========================================================================================================================================================================
#Improting necessary modules
#===========================================================================================================================================================================

#Numpy
import numpy as np
import scipy

#===========================================================================================================================================================================
#Module Functions
#===========================================================================================================================================================================

def computeFitRsq(fit):
	
	"""Computes R-squared values for fit object.
	
	R-squared values contain:
		
		* Mean R-squared value over all ROIs included in fit, stored in ``fit.MeanRsq``.
		* Product of R-squared value over all ROIs included in fit, stored in ``fit.Rsq``.
		* R-squared value for each ROIs included in fit, stored in ``fit.RsqBuROI``.
		
	Args:
		fit (pyfrp.subclasses.pyfrp_fit.fit): Fit object.
		
	Returns:
		pyfrp.subclasses.pyfrp_fit.fit: Updated fit object.
	
	"""
	
	Rsqs=[]
	
	#Compute Rsqs for regions used for fitting	
	for i in range(len(fit.dataVecsFitted)):
		r=Rsq(fit.dataVecsFitted[i],fit.fittedVecs[i])
		fit.RsqByROI[fit.ROIsFitted[i].name]=r
		Rsqs.append(r)
	
	fit.MeanRsq=np.mean(Rsqs)	
	fit.Rsq=np.prod(Rsqs)
		
	return fit

def Rsq(data,x):
	
	r"""Computes R-squared values for fit series to data series.
	
	R-squared value is being computed as 
	
	.. math:: R^2 = 1 - \frac{\sum\limits_i (x_i - d_i)^2}{\sum\limits_i (d_i - \bar{d} )^2}
	
	Args:
		x (numpy.ndarray) Fit series.
		data (numpy.ndarray): Data series.
		
	Returns:
		float: R-squared value.
	
	"""
	
	#Make numpy array to avoid list substraction problem
	data=np.asarray(data)
	x=np.asarray(x)
	
	#Compute ssd between x and data
	ssd=computeSSD(data,x)
	
	#Compute mean data
	meanData=np.mean(data)
	
	#Get derivation of datapoints from mean
	SStot=sum((data-meanData)**2)
	
	#Calculate RSq value
	Rsq=1-ssd/SStot
	
	return Rsq

def computeSSD(data,x):
	
	r"""Computes sum of squared differences (SSD) of fit series to data series.
	
	The SSD is computed by
	
	.. math:: SSD = \sum\limits_i (x_i - d_i)^2
	
	Args:
		x (numpy.ndarray) Fit series.
		data (numpy.ndarray): Data series.
		
	Returns:
		float: SSD.
	"""
	
	return sum((data-x)**2)

def parameterStats(x):
	
	r"""Returns mean, standard deviation and standard error of array.
	
	Note that standard error is computed as 
	
	.. math:: \sigma_n = \frac{\sigma}{\sqrt{n}}
	
	where :math:`n` is the number of samples in ``x`` and :math: :math:`\sigma`
	is the standard deviation over ``x``.
	
	Args:
		x (numpy.ndarray): List of values.
	
	Returns:
		tuple: Tuple containing:
			
			* xMean (float): Mean of x.
			* xStd (float): Standard deviation of x.
			* xSterr (float): Standard error of x.
			
	"""
	
	return np.mean(x), np.std(x), np.std(x)/np.sqrt(len(x))
	
def overlapSubSampleSelect(d,n,k):
	
	r"""Takes subsamples of size ``n`` that overlap on both 
	sides by ``k`` points out of d.
	
	Algorithm collects snippets of d from :math:`j(n-k)` to
	:math:`(j+1)n-jk`, where :math:`j` is the counter for the
	subsamples.
	
	Args:
		d (numpy.ndarray): Data vector.
		n (int): Size of subsamples.
		k (int): Overlap.
		
	Returns:
		list: List of ``numpy.ndarray`` of overlapping subsamples.
		
	"""
	
	j=0
	ds=[]
	for i in range(len(d)):
		try:
			dnew=d[j*(n-k):(j+1)*n-j*k]
		except IndexError:
			dnew=d[j*(n-k):]
		ds.append(dnew)
		
		if (j+1)*n-j*k>len(d):
			break
		
		j=j+1
		
	return ds	

def overlapSubSampleError(d,n,k):
	
	r"""Computes error between overlapping subsamples.
	
	Error is calculated by
	
	.. math:: \left|\frac{\bar{d_i}+\epsilon}{\bar{d_j}+\epsilon}\right|,
	
	where :math:`i,j \in {1,..,\frac{N}{n-k}}` and :math:`\epsilon` is some 
	offset to avoid singularties.
	
	.. note:: The resulting error matrix is symmetric.
	
	Args:
		d (numpy.ndarray): Data vector.
		n (int): Size of subsamples.
		k (int): Overlap.
		
	Returns:
		numpy.ndarray: Error matrix.
	
	"""
	
	ds=overlapSubSampleSelect(d,n,k)
	
	dMean=[]
	for dnew in ds:
		dMean.append(np.mean(dnew))
	
	dError=np.zeros((len(dMean),len(dMean)))
	for i in range(len(dMean)):
		for j in range(len(dMean)):
			dError[i,j]=abs((dMean[i]+1E-10)/(dMean[j]+1E-10+1E-11)-1)
		
	return dError	
	
def selectDataByOverlapSubSample(d,n,k,thresh,debug=False):
	
	"""Selects data vector based on overlapping
	subsampling and simple thresholding.
	
	This algorithm combines local derivatives with global changes
	and filters both datasets that have large local changes as well
	as large global changes. However, taking means over subsamples
	prevents neglecting data sets that have short peaks over only 1
	or 2 time points, such as bubbles etc.
	
	Args:
		d (numpy.ndarray): Data vector.
		n (int): Size of subsamples.
		k (int): Overlap.
		thresh (float): Selecting threshold.
		
	Keyword Args:
		debug (bool): Print debugging messages.
		
	Returns:
		bool: True if data set should be neglected.
	
	"""

	dError=overlapSubSampleError(d,n,k)		
	
	idxs=np.where(dError>thresh)
	
	if debug:
		
		if len(dError[idxs].flatten())>0:
			for i in range(len(idxs[0])):
				print "Subsamples ", idxs[0][i],idxs[1][i], " generate error ", dError[idxs[0][i],idxs[1][i]]
			
	return len(dError[idxs].flatten())>0

def computeLogLikelihood(fit,ROIs=None,neg=True,sigma=1):
	
	r"""Computes log-likelihood of fit assuming
	normal distribution of data around fit.
	
	Generally we assume that the data is distributed normally around
	the fitted model, that is
	
	.. math:: (m-d) \sim \mathcal{N}(0,\sigma)
	
	Assuming that all measurements are independent, this results in 
	a likelihood function of
	
	.. math:: \prod\limits_{j=1}^{n} \frac{1}{\sigma \sqrt{2\pi}} \exp\left(-\frac{(m_j-d_j)^2}{2\sigma^2}\right) 
	
	resulting in a log-likelihood function of
	
	.. math:: - \frac{n}{2} \log (2\pi) - \frac{\sum\limits_{j=1}^{n} (m_j-d_j)^2 }{2\sigma^2}
	
	Function allows to compute both negative and positive log-likelihood by ``neg`` flag.
	
	If the standard deviation :math:`sigma` is unknown, but the log-likelihood is
	only needed for model comparison reasons over the same dataset, all `sigma` dependent
	terms can be ignored. Thus this function returns both SSD and log-likelihood.
	
	.. note:: If ``ROIs=None``, will use all ROIs defined in ``fit.ROIsFitted``. If ``ROIs`` are
	   specified, but not in ``fit.ROIsFitted``, will simply skip them.
	
	Args:
		fit (pyfrp.subclasses.pyfrp_fit.fit): Fit object.
		
	Keyword Args:	
		ROIs (list): List of ROIs to be considered for computation.
		neg (bool): Compute negative log-likehood.
		sigma (float): Standard deviation of normal distribution if known.
		
	Returns:
		tuple: Tuple containing:
		
			* sign (float): Sign of likelihood function (-1/1).
			* SSD (float): SSD of fit over all ROIs specified in ROIs.
			* logLL (float): Log-likehood function at ``sigma=sigma``
	
	"""
	
	#Recompute SSD
	ssds=[]
	for i,r in enumerate(fit.ROIsFitted):
		if ROIs!=None:
			if r not in ROIs:
				continue
		
		ssds.append(computeSSD(fit.dataVecsFitted[i],fit.fittedVecs[i]))
	
	SSD=sum(ssds)
	
	#Compute log likelihood
	n = len(fit.dataVecsFitted[0])
	sign = (2*int(neg)-1)
	const = n/2.*np.log(2*np.pi)
	
	logLL=sign * ( const + SSD/(2*sigma))
	
	return sign,SSD,logLL

def computeAIC(fit,ROIs=None,sigma=1,fromSSD=True):
	
	r"""Computes Akaike Information Criterion of fit.
	
	The AIC is defined by
	
	.. math:: AIC= 2k - 2\log(L),
	
	:math:`k` is the number of free parameters of the model used in 
	``fit`` and :math:`L` is the maximum likelihood function of the
	model, see also :py:func:`computeLogLikelihood`.
	
	Since generally the AIC is only used for model comparison and a normal
	distribution of the data around the model is assumed, one simply
	can use the SSD as a log-likelihood function. ``fromSSD`` controls if
	just the SSD is used. This is also useful, since :math:`sigma` is not 
	necessarily known if the objective function of the fit was not a
	maximum likelihood function.
	
	.. note:: If ``ROIs=None``, will use all ROIs defined in ``fit.ROIsFitted``. If ``ROIs`` are
	   specified, but not in ``fit.ROIsFitted``, will simply skip them.
	   
	If the AIC or the AICc should be used can be determined using :py:func:`useAIC`.
	
	Args:
		fit (pyfrp.subclasses.pyfrp_fit.fit): Fit object.
		
	Keyword Args:	
		ROIs (list): List of ROIs to be considered for computation.
		sigma (float): Standard deviation of normal distribution if known.
		fromSSD (bool): Simply use SSD as maximum likelihood.
		
	Returns:
		float: AIC of fit.
	
	"""
		
	AIC=2*(fit.getNParmsFitted())+2*computeLogLikelihood(fit,ROIs=ROIs)[2-int(fromSSD)]
	
	return AIC

def computeCorrAIC(fit,ROIs=None,sigma=1,fromSSD=True):
	
	r"""Computes corrected Akaike Information Criterion of fit.
	
	The AICc is defined by
	
	.. math:: AICc= AIC + \frac{2k(k+1)}{n-k-1},
	
	where :math:`n` is the number of datapoints used for the fit and 
	:math:`k` is the number of free parameters. For the computation of the
	AIC, please refer to the documentation of :py:func:`computeAIC`.
	
	.. note:: If ``ROIs=None``, will use all ROIs defined in ``fit.ROIsFitted``. If ``ROIs`` are
	   specified, but not in ``fit.ROIsFitted``, will simply skip them.
	   
	If the AIC or the AICc should be used can be determined using :py:func:`useAIC`.
	
	Args:
		fit (pyfrp.subclasses.pyfrp_fit.fit): Fit object.
		
	Keyword Args:	
		ROIs (list): List of ROIs to be considered for computation.
		sigma (float): Standard deviation of normal distribution if known.
		fromSSD (bool): Simply use SSD as maximum likelihood.
		
	Returns:
		float: AICc of fit.
	
	"""
	
	n = len(fit.dataVecsFitted[0])
	AIC=computeAIC(fit,ROIs=ROIs,sigma=sigma,fromSSD=fromSSD)
	k=fit.getNParmsFitted()
	
	corrAIC=AIC + 2*k*(k+1)/(n-k-1)
	
	return corrAIC
	
def useAIC(n,k):

	r"""Returns if corrected or not corrected version of the 
	AIC should be used.
	
	Rule of thumb is that AIC should be used if
	
	.. math:: \frac{n}{k}>40,
	
	where :math:`n` is the number of datapoints used for the fit and 
	:math:`k` is the number of free parameters.
	
	Returns:
		bool: ``True``, use AIC, ``False`` use AICc.
	
	"""
	
	return n/k>40

def compareFitsByAIC(fits,ROIs=None,sigma=1,fromSSD=True,thresh=None):
	
	r"""Computes AIC and Akaike weights for all fits in a list
	and returning best models given certain criteria.
	
	For the computation of the AIC see :py:func:`computeAIC` and 
	the computation of the Akaike weights :py:func:`computeAkaikeWeights`.
	
	If threshold ``thresh=None``, then will select model with maximum
	Akaike weight as best model, that is, the model with the highest likelihood
	of being the best model.
	If ``thresh`` is given, will return list of acceptable models based on
	
	.. math:: w_{\mathrm{max}}-w_i<thresh,
	
	that is, all models that are within a given range of probability of
	the most likely one.
	
	Args:
		fits (list): List of fit objects.
		
	Keyword Args:	
		ROIs (list): List of ROIs to be considered for computation.
		sigma (float): Standard deviation of normal distribution if known.
		fromSSD (bool): Simply use SSD as maximum likelihood.
		thresh (float): Probability range for model selection.
	
	Returns:
		tuple: Tuple containing:
		
			* AICs (list): List of AIC values of the respective fits.
			* deltaAICs (numpy.ndarray): List of Akaike difference values of the respective fits.
			* weights (numpy.ndarray): List of Akaike difference weights of the respective fits.
			* acc (list): List of acceptable fits by model selection.
			* ks (list): List of number of parameters fitted of the respective fits.
			* ns (list): List of number of datapoints fitted of the respective fits.
			
	"""
	
	AICs=[]
	ks=[]
	ns=[]
	for fit in fits:
		AICs.append(computeAIC(fit,ROIs=ROIs,sigma=1,fromSSD=fromSSD))
		ks.append(fit.getNParmsFitted())
		ns.append(len(fit.dataVecsFitted[0]))
		
	deltaAIC=np.asarray(AICs)-min(AICs)	
	
	weights = computeAkaikeWeights(AICs)
	
	acc=[]
	if thresh==None:
		
		idx=list(weights).index(max(weights))
		
		acc=[fits[idx]]
	
	else:
		relWeights=abs(weights-max(weights))
		for i in np.where(list(relWeights<thresh))[0]:
			acc.append(fits[i])
	
	
	return AICs, deltaAIC,weights, acc,ks,ns

def compareFitsByCorrAIC(fits,ROIs=None,sigma=1,fromSSD=True,thresh=None):
	
	r"""Computes AICc and Akaike weights for all fits in a list
	and returning best models given certain criteria.
	
	For the computation of the corrected AIC see :py:func:`computeCorrAIC` and 
	the computation of the Akaike weights :py:func:`computeAkaikeWeights`.
	
	If threshold ``thresh=None``, then will select model with maximum
	Akaike weight as best model, that is, the model with the highest likelihood
	of being the best model.
	If ``thresh`` is given, will return list of acceptable models based on
	
	.. math:: w_{\mathrm{max}}-w_i<thresh,
	
	that is, all models that are within a given range of probability of
	the most likely one.
	
	Args:
		fits (list): List of fit objects.
		
	Keyword Args:	
		ROIs (list): List of ROIs to be considered for computation.
		sigma (float): Standard deviation of normal distribution if known.
		fromSSD (bool): Simply use SSD as maximum likelihood.
		thresh (float): Probability range for model selection.
	
	Returns:
		tuple: Tuple containing:
		
			* AICs (list): List of AICc values of the respective fits.
			* deltaAICs (numpy.ndarray): List of Akaike difference values of the respective fits.
			* weights (numpy.ndarray): List of Akaike difference weights of the respective fits.
			* acc (list): List of acceptable fits by model selection.
			* ks (list): List of number of parameters fitted of the respective fits.
			* ns (list): List of number of datapoints fitted of the respective fits.
	
	"""
	
	AICs=[]
	ks=[]
	ns=[]
	for fit in fits:
		AICs.append(computeCorrAIC(fit,ROIs=ROIs,sigma=1,fromSSD=fromSSD))
		ks.append(fit.getNParmsFitted())
		ns.append(len(fit.dataVecsFitted[0]))
		
	deltaAIC=np.asarray(AICs)-min(AICs)	
	
	weights = computeAkaikeWeights(AICs)
	
	acc=[]
	if thresh==None:
		
		idx=list(weights).index(max(weights))
		acc=[fits[idx]]
	
	else:
		relWeights=abs(weights-max(weights))
		
		for i in np.where(list(relWeights<thresh))[0]:
			acc.append(fits[i])
	
	return AICs, deltaAIC,weights, acc,ks,ns

	
def computeAkaikeWeights(AICs):
	
	r"""Computes Akaike weights for a list of 
	AIC values.
	
	Akaike weightsare given by:
	
	.. math:: w_i = \frac{\exp\left(\frac{-\Delta_{i}}{2}\right)}{\sum\limits_{i=1}^{N} \exp\left(\frac{- \Delta_{i}}{2}\right)},
	
	where :math:`\Delta_{i}` is the Akaike difference of model :math:`i` computed by
	
	.. math:: \Delta_{i} = AIC_i - AIC_{\mathrm{min}},
	
	and :math:`N` is the total number of models considered. Note that
	
	.. math:: \sum\limits_{i=1}^{N} \exp\left(\frac{- \Delta_{i}}{2}\right) = 1.
	
	Args:
		AICs (list): List of AIC values.
		
	Returns:
		np.ndarray: Corresponding Akaike weights.
	
	
	"""
	
	deltaAIC=np.asarray(AICs)-min(AICs)	
	
	weights=np.exp(-deltaAIC/2.)/sum(np.exp(-deltaAIC/2.))
	
	
	return weights

def wilcoxonTest(x,y,zero_method='wilcox', correction=False,printOut=True):
	
	"""Performs wilcoxon test between two groups.
	
	See also https://docs.scipy.org/doc/scipy-0.14.0/reference/generated/scipy.stats.wilcoxon.html
	
	Args:
		x (list): First group.
		y (list): Second group.
		
	Keyword Args:
		zero_method (str): Treatment of zeros.
		correction (bool): Apply continuety correction.
		printOut (bool): Print out results.
	
	Returns:
		tuple: Tuple containing:
		
			* stat (float): Sum of ranks.
			* val (float): p-Value.
			
	"""
	
	stat,pval=scipy.stats.wilcoxon(x, y=y, zero_method=zero_method, correction=correction)
	
	if printOut:
		print "Results of Wilcoxon-Test:"
		print "p-Value: ", pval
		print "Wilcoxon-Statistics:", stat
	
	return stat,pval

def mannWhitneyUTest(x,y,printOut=True):
	
	"""Performs Mann-Whitney-U test between two groups.
	
	See also https://docs.scipy.org/doc/scipy-0.14.0/reference/generated/scipy.stats.mannwhitneyu.html#scipy.stats.mannwhitneyu
	
	Args:
		x (list): First group.
		y (list): Second group.
		
	Keyword Args:
		printOut (bool): Print out results.
	
	Returns:
		tuple: Tuple containing:
		
			* stat (float): Mann-Whitney statistics.
			* val (float): p-Value.
			
	"""
	
	stat,pval=scipy.stats.mannwhitneyu(x, y)
	
	if printOut:
		print "Results of Mann-Whitney-U-Test:"
		print "p-Value: ", pval
		print "U-Statistics:", stat
	
	return stat,pval

def tTestStandard(x,y,printOut=True):
	
	"""Performs standard t-test between two groups.
	
	See also https://docs.scipy.org/doc/scipy-0.14.0/reference/generated/scipy.stats.ttest_ind.html#scipy.stats.ttest_ind
	
	Args:
		x (list): First group.
		y (list): Second group.
		
	Keyword Args:
		printOut (bool): Print out results.
	
	Returns:
		tuple: Tuple containing:
		
			* stat (float): Test statistics.
			* val (float): p-Value.
			
	"""
	
	stat,pval=scipy.stats.ttest_ind(x, y, equal_var=True)
	
	if printOut:
		print "Results of Standard t-Test:"
		print "p-Value: ", pval
		print "Statistics:", stat
		
	return stat,pval
	
def tTestWelch(x,y,printOut=True):
	
	"""Performs Welch's t-test between two groups.
	
	See also https://docs.scipy.org/doc/scipy-0.14.0/reference/generated/scipy.stats.ttest_ind.html#scipy.stats.ttest_ind
	
	Args:
		x (list): First group.
		y (list): Second group.
		
	Keyword Args:
		printOut (bool): Print out results.
	
	Returns:
		tuple: Tuple containing:
		
			* stat (float): Test statistics.
			* val (float): p-Value.
			
	"""
	
	stat,pval=scipy.stats.ttest_ind(x, y, equal_var=False)
	
	if printOut:
		print "Results of Welch's t-Test:"
		print "p-Value: ", pval
		print "Statistics:", stat	
	
	return stat,pval
	
def shapiroTest(x,printOut=True):
	
	"""Performs shapiro test to test group for normality.
	
	See also https://docs.scipy.org/doc/scipy-0.14.0/reference/generated/scipy.stats.shapiro.html#scipy.stats.shapiro
	
	Args:
		x (list): Data.
		
	Keyword Args:
		printOut (bool): Print out results.
	
	Returns:
		tuple: Tuple containing:
		
			* stat (float): Test statistic.
			* val (float): p-Value.
			
	"""

	stat,pval=scipy.stats.sharipo(x)
	
	if printOut:
		print "Results of Sharipo-Test:"
		print "p-Value: ", pval
		print "Statistics:", stat
	
	return stat,pval

	
	
	
	
	
	
	


#=====================================================================================================================================
#Copyright
#=====================================================================================================================================

#Copyright (C) 2014 Alexander Blaessle, Patrick Mueller and the Friedrich Miescher Laboratory of the Max Planck Society
#This software is distributed under the terms of the GNU General Public License.

#This file is part of PyFRAP.

#PyFRAP is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

#===========================================================================================================================================================================
#Module Description
#===========================================================================================================================================================================

#Main molecule class saving all important data, parameters and results for particular molecule.
	
#===========================================================================================================================================================================
#Improting necessary modules
#===========================================================================================================================================================================

#Numpy
import numpy as np

#PyFRAP Modules
from pyfrp.modules import pyfrp_misc_module
from pyfrp.modules import pyfrp_IO_module
from pyfrp.modules import pyfrp_stats_module

#PyFRAP Classes
import pyfrp_embryo

#Standard packages
import os


#---------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#Molecule object

class molecule:
	
	"""Molecule class, collecting information about a series of FRAP experiments.
	
	The main purpose of the molecule class is to gather and summarize multiple FRAP experiments.
	Embryo objects are stored in a ``embryos`` list. From those embryo objects, fit objects can be
	added to the ``selFits`` list to then be summarized to calculate measurement statistics.
	Fits can be forced to overlap in a set of parameters defined in ``crucialParameters``.
	
	"""
	
	
	#Creates new molecule object
	def __init__(self,name):
		
		#Data
		self.name=name
		self.embryos=[]
		
		#Results
		self.selFits=[]
		self.DOptMu=None
		self.DOptPx=None
		self.DOptPxStd=None
		self.DOptPxSterr=None
		self.prodOpt=None
		self.degrOpt=None
		self.MeanRsq=None
		self.Rsq=None
		self.DMuStd=None
		self.prodStd=None
		self.degrStd=None
		self.DMuStErr=None
		self.prodStErr=None
		self.degrStErr=None
		
		self.crucialParameters=["equOn","fitPinned","fitProd","fitDegr","LBD","LBProd","LBDegr","UBD","UBProd","UBDegr"]
					
	def addEmbryo(self,embryo):
		
		"""Appends embryo object to ``embryos`` list.
		
		Args:
			embryo (pyfrp.subclasses.pyfrp_embryo): Embryo to append.
		
		Returns:
			list: Updated pyfrp.subclasses.pyfrp_molecule.embryos list
		
		"""
		
		self.embryos.append(embryo)
		return self.embryos
	
	def replaceEmbryo(self,embryo,name=""):
		
		"""Replaces embryo with same name that ``embryo`` in molecule's ``embryos`` list.
		
		If no embryo with the same name exists, will simply add embryo. If ``name`` is given,
		will try to replace embryo with name ``name`` in ``embryos`` list.
		
		Args:
			embryo (pyfrp.subclasses.pyfrp_embryo): New embryo to be inserted.
			
		Keyword Args:
			name (str): Optional name of embryo embryo to be replaced.

		Returns:
			bool: True if anything was replaced, False if anything was appended.
			
		"""
		
		for i,emb in enumerate(self.embryos):
			if name!="":
				if emb.name==embryo.name:
					self.embryos[i]=embryo
					return True
			else:
				if emb.name==name:
					self.embryos[i]=embryo
					return True
			
		self.addEmbryo(embryo)
		
		return False
		
	def newEmbryo(self,name):
		
		"""Creates new embryo and appends it to ``embryos`` list.
		
		Args:
			name (str): Name of new embryo.
		
		Returns:
			pyfrp.subclasses.pyfrp_embryo: New embryo object.
		
		"""
		
		emb=pyfrp_embryo.embryo(name)
		self.embryos.append(emb)
		return self.embryos[-1]
		
	def removeEmbryo(self,embryo):
			
		"""Removes embryo object from ``embryos`` list.
		
		Args:
			embryo (pyfrp.subclasses.pyfrp_embryo): Embryo object.
		
		Returns:
			list: Updated pyfrp.subclasses.pyfrp_molecule.embryos list
		
		"""
		
		self.embryos.remove(embryo)
		return self.embryos		
	
	def save(self,fn=None):
		
		"""Saves molecule to pickle file.
		
		.. note:: If ``fn`` is not specified, will assume ``fn=self.name``.
		
		Keyword Args:
			fn (str): Molecule file name.
		
		Returns:
			str: Filename of molecule file.
		
		"""
		
		if fn==None:
			fn=self.name+".mol"
		
		pyfrp_IO_module.saveToPickle(self,fn=fn)	
		
		return fn
	
	def saveExtract(self,fn=None,copyMeshFiles=True,debug=False):
		
		"""Saves molecule to pickle file in compressed version by doing:
		
			* Extracts embryos in ``embryos`` list into seperate pickled files.
			* Clears all attributes of embryo objects
			* Saves molecule file
			
		This function is really useful if molecule file size gets out-of-hand.	
		
		.. note:: Embryo files will be saved in path/to/moculefile/moleculename/ .
		
		.. note:: If ``fn`` is not specified, will assume ``fn=self.name``.
		
		Keyword Args:
			fn (str): Molecule file name.
			copyMeshFiles (bool): Copy meshfiles to embryo file destination.
			debug (bool): Print out debugging messages.
			
		Returns:
			bool: True if success, False else.
		
		"""
		
		if not self.checkEmbryoNames():
			printError("Embryo names are not distinct, will not save.")
			return False
		
		if fn==None:
			fn=self.name+".mol"
		
		
		b=self.extractEmbryos2Files(fn=fn.replace('.mol','')+"_embryos",copyMeshFiles=copyMeshFiles,debug=debug)
		
		if not b:
			printError("Something went wrong extracting embryos, will not continue saving.")
			return False
		
		self.clearAllEmbryos()
		self.save(fn=fn)
		
		return True
		
		
	
	def extractEmbryos2Files(self,fn="",copyMeshFiles=True,debug=False):
		
		"""Extracts embryos in ``embryos`` list into seperate pickled files.
		
		.. note:: Will create folder ``fn`` if non-existent. If ``fn`` is not specified, will assume ``fn='embryoFiles/'`` .
		
		Keyword Args:
			fn (str): Path of folder where to save embryo files.
			copyMeshFiles (bool): Copy meshfiles to embryo file destination.
			debug (bool): Print out debugging messages.
			
		Returns:
			bool: True if success, False else.
		
		"""
		
		if fn=="":
			try:
				printWarning('fn not specified, will assume fn=embryoFiles/')
				os.system("mkdir -v embryoFiles/")
				fn="embryoFiles/"
			except OSError:
				pass
		
		if not os.path.isdir(fn):
			printWarning(fn+" does not  exist, will try to create it.")
			try:
				os.system("mkdir -v " +fn)
			except OSError:
				printError("Could not create " + fn+ ". Will not extract." )
				return False
		
		print "in mol.extract:", fn
		
		for embryo in self.embryos:
			embryo.save(fn=fn+embryo.getName()+".emb",copyMeshFiles=copyMeshFiles,debug=debug)
		
		return True
		
	def clearAllEmbryos(self):
		
		"""Replaces all attribute values of each embryo in `embryos` list with ``None``, except ``name``.
		
		Useful if embryos are seperated and molecule file needs to be compressed.
		
		.. note:: Embryos should have all different names, so there will not be any missassignment when reimporting embryo files.
		
		Returns:
			bool: True if success, False else.
		
		"""
		
		b=True
		for embryo in self.embryos:
			bnew=embryo.clearAllAttributes()
			b=b+bnew
		
		return b
	
	def checkEmbryoNames(self):
		
		"""Check if all embryos in ``embryos`` list have different names.
		
		Returns:
			bool: True if all different, False else.	
		"""
		
		names=pyfrp_misc_module.objAttrToList(self.embryos,'name')
		b=True
		for name in names:
			if names.count(name)>1:
				printWarning('There is more than one embryo called '+ name + ' in molecule ' + self.name)
				b=False
		return b		
		
	def updateVersion(self):
		
		"""Updates molecule file to current version, making sure that it possesses
		all attributes.
		
		Creates a new molecule object and compares ``self`` with the new molecule file.
		If the new molecule object has a attribute that ``self`` does not have, will
		add attribute with default value from the new molecle file.
		
		.. note:: Will also update all subobject, making sure that embryo and fit objects are up-to-date.
		
		Returns:
			pyfrp.subclasses.pyfrp_molecule: ``self``
			
		"""
		
		#Create temporarly a blank molecule file
		moltemp=molecule("temp")
		
		#Update molecule file
		pyfrp_misc_module.updateObj(moltemp,self)
		
		#Update all embryos and fits
		for emb in self.embryos:
			
			emb.updateVersion()
			
			for fit in emb.fits:
				fit.updateVersion()
		
		return self
		
	def getEmbryoByName(self,s):
		
		"""Returns embryo with name ``s`` from ``embryos`` list, otherwise ``False``. 
		"""
		
		for emb in self.embryos:
			if str(s) == emb.name:
				return emb
		return False
	
	def setName(self,n):
		
		"""Sets molecule's name.
		
		Args:
			n (str): New name.
		
		Returns:
			str: Name.
			
		"""
		
		self.name=n
		return self.name
	
	def getName(self):
	
		"""Returns molecule's name.
		
		Returns:
			str: Name.
			
		"""
		
		return self.name
	
	
	
	def sumUpResults(self,sameSettings=False):
		
		"""Sums up results from all fits in ``selFits`` list.
		
		Keyword Args:
			sameSettings (bool): Fits must overlap in parameters defined in ``crucialParameters``.
		
		Returns:
			bool: True if success, False else.	
		"""
		
		if sameSettings:
			
			lastFit=self.selFits[0]
			
			#Check that all fits have roughly the same parameters
			for fit in self.selFits:
				
				#Check if fit is fitted
				if not fit.isFitted():
					printError("Cannot average fits, since fits " + fit.name + "has not been fitted yet.")
					return False
				
				same,different,notInBoth=pyfrp_misc_module.compareObjAttr(lastFit,fit)
				
				for item in self.crucialParameters:   
					if item in different.keys():
						printError("Cannot average fits, since fits " + lastFit.name + " and " + fit.name + "do not have the same value for " + item +". However, this parameter is marked as crucial.")
						return False
					elif item in notInBoth:
						printError("Cannot average fits, since one of the fits " + lastFit.name + " and " + fit.name + " lacks the parameter " + item +".")
						return False
		
		#Compute Statistics
		self.DOptMu,self.DOptMuStd,self.DOptMuSterr=pyfrp_stats_module.parameterStats(pyfrp_misc_module.objAttrToList(self.selFits,"DOptMu"))
		self.DOptPx,self.DOptPxStd,self.DOptPxSterr=pyfrp_stats_module.parameterStats(pyfrp_misc_module.objAttrToList(self.selFits,"DOptPx"))
		self.prodOpt,self.prodOptStd,self.prodOptSterr=pyfrp_stats_module.parameterStats(pyfrp_misc_module.objAttrToList(self.selFits,"prodOpt"))
		self.degrOpt,self.degrOptStd,self.degrOptSterr=pyfrp_stats_module.parameterStats(pyfrp_misc_module.objAttrToList(self.selFits,"degrOpt"))
		
		self.Rsq=np.mean(pyfrp_misc_module.objAttrToList(self.selFits,"Rsq"))
		self.MeanRsq=np.mean(pyfrp_misc_module.objAttrToList(self.selFits,"MeanRsq"))
			
		return	True
		
	def printResults(self):
		
		"""Prints results summarized in :py:func:`pyfrp.subclasses.pyfrp_molecule.sumUpResults`."""
		
		printObjAttr('name',self)
		printObjAttr('DOptMu',self)
		printObjAttr('DOptMuStd',self)
		printObjAttr('DOptMuSterr',self)
		printObjAttr('DOptPx',self)
		printObjAttr('DOptPxStd',self)
		printObjAttr('DOptPxSterr',self)
		printObjAttr('prodOpt',self)
		printObjAttr('prodOpt',self)
		printObjAttr('prodOptStd',self)
		printObjAttr('prodOptSterr',self)
		printObjAttr('degrOpt',self)
		printObjAttr('degrOptStd',self)
		printObjAttr('degrOptSterr',self)
		printObjAttr('Rsq',self)
		printObjAttr('MeanRsq',self)
		
	def getFitParm(self,parm):
		
		"""Collects all values of parameter from all selected fits of molecule.
		
		Args:
			parm (str): Name of parameter.
			
		Returns:
			list: List of values.
			
		"""	
		
		ps=[]
		for fit in self.selFits:
			
			try:
				ps.append(getattr(fit,parm))
			except AttributeError:
				printError("Fit " +fit.name + "does not have an attribute with name = ", parm )
			
		return ps	
		
	def getDOptMus(self):	
		
		"""Collects all values of DOptMu from all selected fits of molecule.
	
		Returns:
			list: List of diffusion rates.
			
		"""	
		
		return self.getFitParm("DOptMu")
	
	


#=====================================================================================================================================
#Copyright
#=====================================================================================================================================

#Copyright (C) 2014 Alexander Blaessle, Patrick Mueller and the Friedrich Miescher Laboratory of the Max Planck Society
#This software is distributed under the terms of the GNU General Public License.

#This file is part of PyFRAP.

#PyFRAP is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

#===========================================================================================================================================================================
#Module Description
#===========================================================================================================================================================================

#PyQT Dialogs for fit class
#(1) 

#===========================================================================================================================================================================
#Importing necessary modules
#===========================================================================================================================================================================

#QT
from PyQt4 import QtGui, QtCore

#PyFRAP GUI classes
import pyfrp_gui_basics

#PyFRAP modules
from pyfrp.modules.pyfrp_term_module import *
from pyfrp.modules import pyfrp_img_module
from pyfrp.modules import pyfrp_misc_module

#Numpy/Scipy
import numpy as np

#Misc 
import os


#===================================================================================================================================
#Dialog for editing simulation settings
#===================================================================================================================================

class fitSettingsDialog(pyfrp_gui_basics.basicSettingsDialog):
	
	def __init__(self,fit,parent):
		
		super(fitSettingsDialog,self).__init__(parent)
		
		self.fit = fit
		
		#Labels
		self.lblName = QtGui.QLabel("Name:", self)
		
		self.lblOptMeth = QtGui.QLabel("Optimization Method:", self)
		self.lblMaxFun = QtGui.QLabel("Max. Function Calls:", self)
		self.lblOptTol = QtGui.QLabel("Precision goal:", self)
		
		self.lblX0D = QtGui.QLabel("x0(D):", self)
		self.lblX0Prod = QtGui.QLabel("x0(production):", self)
		self.lblX0Degr = QtGui.QLabel("x0(degration):", self)
		
		self.lblBoundsD = QtGui.QLabel("<= D <=", self)
		self.lblBoundsProd = QtGui.QLabel("<= production <=", self)
		self.lblBoundsDegr = QtGui.QLabel("<= degration <=", self)
		
		self.lblKinetic = QtGui.QLabel("Kinetic Timescale:", self)
		self.lblSaveTrack = QtGui.QLabel("Save Track:", self)
		
		self.lblEquOn = QtGui.QLabel("Equalization:", self)
		self.lblFitPinned = QtGui.QLabel("Fit Pinned:", self)
		
		self.lblFitProd = QtGui.QLabel("Fit Production:", self)
		self.lblFitDegr = QtGui.QLabel("Fit Degradation:", self)
		
		self.lblFitCutOffT = QtGui.QLabel("Fit Cut-Off:", self)
		self.lblCutOffT = QtGui.QLabel("t(Cut-Off):", self)
		
		boldfont = QtGui.QFont()
		boldfont.setBold(True)
		
		self.lblHeadBounds = QtGui.QLabel("Bounds:", self)
		self.lblHeadBounds.setFont(boldfont)
		
		self.lblHeadX0 = QtGui.QLabel("Initial Guess:", self)
		self.lblHeadX0.setFont(boldfont)
		
		self.lblHeadGeneral = QtGui.QLabel("General:", self)
		self.lblHeadGeneral.setFont(boldfont)
		
		self.lblHeadROI = QtGui.QLabel("ROIs Fitted:", self)
		self.lblHeadROI.setFont(boldfont)
		
		self.lblHeadOptions = QtGui.QLabel("Fit Options:", self)
		self.lblHeadOptions.setFont(boldfont)
		
		#LineEdits
		self.qleName = QtGui.QLineEdit(self.fit.name)
		
		self.qleMaxFun = QtGui.QLineEdit(str(self.fit.maxfun))
		self.qleOptTol = QtGui.QLineEdit(str(self.fit.optTol))
		
		self.qleX0D = QtGui.QLineEdit(str(self.fit.x0[0]))
		self.qleX0Prod = QtGui.QLineEdit(str(self.fit.x0[1]))
		self.qleX0Degr = QtGui.QLineEdit(str(self.fit.x0[2]))
		
		self.qleLBD = QtGui.QLineEdit(str(self.fit.LBD))
		self.qleLBProd = QtGui.QLineEdit(str(self.fit.LBProd))
		self.qleLBDegr = QtGui.QLineEdit(str(self.fit.LBDegr))
		
		self.qleUBD = QtGui.QLineEdit(str(self.fit.UBD))
		self.qleUBProd = QtGui.QLineEdit(str(self.fit.UBProd))
		self.qleUBDegr = QtGui.QLineEdit(str(self.fit.UBDegr))
		
		self.qleKinetic = QtGui.QLineEdit(str(self.fit.kineticTimeScale))

		self.qleCutOffT = QtGui.QLineEdit(str(self.fit.cutOffT))
		
		self.doubleValid=QtGui.QDoubleValidator()
		self.intValid=QtGui.QIntValidator()
	
		self.qleX0D.setValidator(self.doubleValid)
		self.qleX0Prod.setValidator(self.doubleValid)
		self.qleX0Degr.setValidator(self.doubleValid)
		
		self.qleKinetic.setValidator(self.intValid)

		self.qleCutOffT.setValidator(self.doubleValid)
			
		self.qleName.editingFinished.connect(self.setName)
		self.qleMaxFun.editingFinished.connect(self.setMaxfun)
		
		self.qleOptTol.editingFinished.connect(self.setOptTol)
		self.qleKinetic.editingFinished.connect(self.setKineticTimeScale)
		
		self.qleLBProd.editingFinished.connect(self.setLBProd)
		self.qleLBDegr.editingFinished.connect(self.setLBDegr)
		self.qleLBD.editingFinished.connect(self.setLBD)
		
		self.qleUBProd.editingFinished.connect(self.setUBProd)
		self.qleUBDegr.editingFinished.connect(self.setUBDegr)
		self.qleUBD.editingFinished.connect(self.setUBD)
		
		self.qleX0Prod.editingFinished.connect(self.setX0Prod)
		self.qleX0Degr.editingFinished.connect(self.setX0Degr)
		self.qleX0D.editingFinished.connect(self.setX0D)
		
		self.qleCutOffT.editingFinished.connect(self.setCutOffT)
		
		#ComboBox
		self.comboMeth = QtGui.QComboBox(self)
		self.comboMeth.addItem("Constrained Nelder-Mead")
		self.comboMeth.addItem("TNC")
		self.comboMeth.addItem("Nelder-Mead")
		self.comboMeth.addItem("L-BFGS-B")
		self.comboMeth.addItem("SLSQP")
		self.comboMeth.addItem("brute")
		self.comboMeth.addItem("BFGS")
		self.comboMeth.addItem("CG")
		
		self.initComboMeth()
		
		self.comboMeth.activated[str].connect(self.setOptMeth)   
		
		#Checkboxes
		self.cbFitProd = QtGui.QCheckBox('', self)
		self.cbFitDegr = QtGui.QCheckBox('', self)
		self.cbFitPinned = QtGui.QCheckBox('', self)
		self.cbEquOn = QtGui.QCheckBox('', self)
		self.cbFitCutOffT = QtGui.QCheckBox('', self)
		self.cbSaveTrack = QtGui.QCheckBox('', self)
	
		self.updateCBs()
		
		self.connect(self.cbFitProd, QtCore.SIGNAL('stateChanged(int)'), self.checkFitProd)
		self.connect(self.cbFitDegr, QtCore.SIGNAL('stateChanged(int)'), self.checkFitDegr)
		self.connect(self.cbFitPinned, QtCore.SIGNAL('stateChanged(int)'), self.checkFitPinned)
		self.connect(self.cbFitCutOffT, QtCore.SIGNAL('stateChanged(int)'), self.checkFitCutOffT)
		self.connect(self.cbEquOn, QtCore.SIGNAL('stateChanged(int)'), self.checkEquOn)
		self.connect(self.cbSaveTrack, QtCore.SIGNAL('stateChanged(int)'), self.checkSaveTrack)
		
		#TreeWigget
		self.ROIList=QtGui.QTreeWidget()
		self.ROIList.setHeaderLabels(["Name"])
		self.ROIList.setColumnWidth(0,100)
		self.updateROIList()
		
		#Buttons
		self.btnAddROI=QtGui.QPushButton('Add')
		self.btnDone.connect(self.btnAddROI, QtCore.SIGNAL('clicked()'), self.addROI)
		
		self.btnRemoveROI=QtGui.QPushButton('Remove')
		self.btnRemoveROI.connect(self.btnRemoveROI, QtCore.SIGNAL('clicked()'), self.removeROI)
		
		#Layout	
		self.grid.addWidget(self.lblHeadGeneral,0,1,1,2,QtCore.Qt.AlignHCenter)
		self.grid.addWidget(self.lblName,1,1)
		self.grid.addWidget(self.lblOptMeth,2,1)
		self.grid.addWidget(self.lblMaxFun,3,1)
		self.grid.addWidget(self.lblOptTol,4,1)
		self.grid.addWidget(self.lblKinetic,5,1)
		self.grid.addWidget(self.lblSaveTrack,6,1)
		
		self.grid.addWidget(self.qleName,1,2)
		self.grid.addWidget(self.comboMeth,2,2)
		self.grid.addWidget(self.qleMaxFun,3,2)
		self.grid.addWidget(self.qleOptTol,4,2)
		self.grid.addWidget(self.qleKinetic,5,2)
		self.grid.addWidget(self.cbSaveTrack,6,2)
		
		self.grid.addWidget(self.lblHeadX0,8,1,1,2,QtCore.Qt.AlignHCenter)
		self.grid.addWidget(self.lblX0D,9,1)
		self.grid.addWidget(self.lblX0Prod,10,1)
		self.grid.addWidget(self.lblX0Degr,11,1)
		
		self.grid.addWidget(self.qleX0D,9,2)
		self.grid.addWidget(self.qleX0Prod,10,2)
		self.grid.addWidget(self.qleX0Degr,11,2)
		
		self.grid.addWidget(self.lblHeadBounds,0,3,1,3,QtCore.Qt.AlignHCenter)
		self.grid.addWidget(self.qleLBD,1,3)
		self.grid.addWidget(self.qleLBProd,2,3)
		self.grid.addWidget(self.qleLBDegr,3,3)
		
		self.grid.addWidget(self.lblBoundsD,1,4,QtCore.Qt.AlignHCenter)
		self.grid.addWidget(self.lblBoundsProd,2,4,QtCore.Qt.AlignHCenter)
		self.grid.addWidget(self.lblBoundsDegr,3,4,QtCore.Qt.AlignHCenter)
		
		self.grid.addWidget(self.qleUBD,1,5)
		self.grid.addWidget(self.qleUBProd,2,5)
		self.grid.addWidget(self.qleUBDegr,3,5)
		
		self.grid.addWidget(self.lblHeadROI,5,3,1,3,QtCore.Qt.AlignHCenter)
		self.grid.addWidget(self.ROIList,6,3,5,3)
		self.grid.addWidget(self.btnAddROI,11,3)
		self.grid.addWidget(self.btnRemoveROI,11,5)
		
		self.grid.addWidget(self.lblHeadOptions,0,6,1,2,QtCore.Qt.AlignHCenter)
		self.grid.addWidget(self.lblEquOn,1,6)
		self.grid.addWidget(self.lblFitPinned,2,6)
		self.grid.addWidget(self.lblFitProd,3,6)
		self.grid.addWidget(self.lblFitDegr,4,6)
		self.grid.addWidget(self.lblFitCutOffT,5,6)
		self.grid.addWidget(self.lblCutOffT,6,6)
		
		self.grid.addWidget(self.cbEquOn,1,7)
		self.grid.addWidget(self.cbFitPinned,2,7)
		self.grid.addWidget(self.cbFitProd,3,7)
		self.grid.addWidget(self.cbFitDegr,4,7)
		self.grid.addWidget(self.cbFitCutOffT,5,7)
		self.grid.addWidget(self.qleCutOffT,6,7)
		
		self.resize(900,700)
		
		self.setWindowTitle("Fit Settings")
		
		self.show()
	
	def validateBound(self,b):
		if b==None or b=="":
			return True
		try:
			c=float(b)
			return True
		except:
			printError("Invalid boundary entered.")
			return False
			
	def updateROIList(self):
		
		self.ROIList.clear()
		for r in self.fit.ROIsFitted:
			QtGui.QTreeWidgetItem(self.ROIList,[r.name])
		return self.ROIList
	
	def initComboMeth(self):
		idx=self.comboMeth.findText(self.fit.optMeth,QtCore.Qt.MatchExactly)
		self.comboMeth.setCurrentIndex(idx)
		
	def updateCBs(self):	
		self.cbEquOn.setCheckState(2*int(self.fit.equOn))
		self.cbFitPinned.setCheckState(2*int(self.fit.fitPinned))
		self.cbFitProd.setCheckState(2*int(self.fit.fitProd))
		self.cbFitDegr.setCheckState(2*int(self.fit.fitDegr))
		self.cbFitCutOffT.setCheckState(2*int(self.fit.fitCutOffT))
		self.cbSaveTrack.setCheckState(2*int(self.fit.saveTrack))
		
	def checkEquOn(self,val):
		self.fit.setEqu(bool(2*val))
	
	def checkFitPinned(self,val):
		self.fit.setFitPinned(bool(2*val))
	
	def checkFitProd(self,val):
		self.fit.setFitProd(bool(2*val))
	
	def checkFitDegr(self,val):
		self.fit.setFitDegr(bool(2*val))
	
	def checkFitCutOffT(self,val):
		self.fit.setFitCutOffT(bool(2*val))
	
	def checkSaveTrack(self,val):
		self.fit.setSaveTrack(bool(2*val))
	
	def setLBD(self):
		text=str(self.qleLBD.text())
		if text=="" or text=='None':
			text=None	
			self.fit.setLBD(text)
		else:
			if self.validateBound(text):
				self.fit.setLBD(float(text))
	
	def setLBProd(self):
		text=str(self.qleLBProd.text())
		if text=="" or text=='None':
			text=None	
			self.fit.setLBProd(text)
		else:
			if self.validateBound(text):
				self.fit.setLBProd(float(text))
	
	def setLBDegr(self):
		text=str(self.qleLBDegr.text())
		if text=="" or text=='None':
			text=None	
			self.fit.setLBDegr(text)
		else:
			if self.validateBound(text):
				self.fit.setLBDegr(float(text))
		
	def setUBD(self):
		text=str(self.qleUBD.text())
		if text=="" or text=='None':
			text=None	
			self.fit.setUBD(text)
		else:
			if self.validateBound(text):
				self.fit.setUBD(float(text))
	
	def setUBProd(self):
		text=str(self.qleUBProd.text())
		if text=="" or text=='None':
			text=None	
			self.fit.setUBProd(text)
		else:
			if self.validateBound(text):
				self.fit.setUBProd(float(text))
	
	def setUBDegr(self):
		text=str(self.qleUBDegr.text())
		if text=="" or text=='None':
			text=None	
			self.fit.setUBDegr(text)
		else:
			if self.validateBound(text):
				self.fit.setUBDegr(float(text))
		
	def setX0D(self):
		text=str(self.qleX0D.text())
		if text=="":
			printWarning("You must give a value for x0.")
			return
		self.fit.setX0D(float(text))
		
	def setX0Prod(self):
		text=str(self.qleX0Prod.text())
		if text=="":
			printWarning("You must give a value for x0.")
			return
		self.fit.setX0Prod(float(text))
		
	def setX0Degr(self):
		text=str(self.qleX0Degr.text())
		if text=="":
			printWarning("You must give a value for x0.")
			return
		self.fit.setX0Degr(float(text))
		
	def setName(self):
		self.fit.setName(str(self.qleName.text()))
		
	def setOptMeth(self):
		self.fit.setOptMeth(str(self.comboMeth.currentText()))
	
	def setKineticTimeScale(self):
		self.fit.setKineticTimeScale(float(str(self.qleKinetic.text())))
	
	def setMaxfun(self):
		self.fit.setMaxfun(int(str(self.qleMaxFun.text())))
		
	def setOptTol(self):
		self.fit.setOptTol(float(str(self.qleOptTol.text())))
		
	def setCutOffT(self):
		self.fit.setCutOffT(float(str(self.qleCutOffT.text())))
	
	def addROI(self):
		
		nameList=pyfrp_misc_module.objAttrToList(self.fit.embryo.ROIs,'name')
		
		selectorDialog = pyfrp_gui_basics.basicSelectorDialog(nameList,self)
		if selectorDialog.exec_():
			selectedROIName = selectorDialog.getItem()
			if selectedROIName==None:
				return
		
		selectedROI=self.fit.embryo.ROIs[nameList.index(selectedROIName)]
		
		if selectedROI not in self.fit.ROIsFitted:
			self.fit.addROI(selectedROI)
		
		self.updateROIList()
		
	def removeROI(self):
		idx=self.ROIList.indexFromItem(self.ROIList.currentItem()).row()
		self.fit.removeROI(self.fit.ROIsFitted[idx])
		self.updateROIList()
		
#===================================================================================================================================
#Dialogs for fitting progress
#===================================================================================================================================

class fittingProgressDialog(pyfrp_gui_basics.waitDialog):
	
	def __init__(self,parent):
		super(fittingProgressDialog,self).__init__(parent)
		
		#Labels
		self.lblName.setText("Fitting in progress...")
		
		#Window title
		self.setWindowTitle('Fitting progress')
		    
		self.show()

class fittingThread(pyfrp_gui_basics.pyfrpThread):
	
	def __init__(self, fit=None, parent=None):
		
		super(fittingThread,self).__init__(parent)
		self.obj=fit
		self.fit=fit
		
			
	def runTask(self,debug=False):
		self.fit.run(debug=debug)
			
			



#=====================================================================================================================================
#Copyright
#=====================================================================================================================================

#Copyright (C) 2014 Alexander Blaessle, Patrick Mueller and the Friedrich Miescher Laboratory of the Max Planck Society
#This software is distributed under the terms of the GNU General Public License.

#This file is part of PyFRAP.

#PyFRAP is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

#===========================================================================================================================================================================
#Module Description
#===========================================================================================================================================================================

"""Simulaton module for PyFRAP toolbox. Handles simulating FRAP experiments and all necessary functions to do so, such as
	
	* Handling initial conditions.
	* Mimicing bleaching effects.
	* Experiment simulation.

"""

#===========================================================================================================================================================================
#Improting necessary modules
#===========================================================================================================================================================================

#PDE Toolbox
from fipy import *

#Numpy/Scipy
import numpy as np
import scipy.interpolate as interp 
import scipy.ndimage.interpolation as ndi

#matplotlib
import matplotlib.pyplot as plt

#Misc
import time
import sys

#PyFRAP Modules
import pyfrp_plot_module 
import pyfrp_integration_module
import pyfrp_misc_module
from pyfrp_term_module import *
import pyfrp_idx_module

#===========================================================================================================================================================================
#Module Functions
#===========================================================================================================================================================================

def simulateReactDiff(simulation,signal=None,embCount=None,showProgress=True,debug=False):
	
	r"""Simulates reaction diffusion equation goverining FRAP experiment.
	
	Performs the following steps:
		
		* Resets ``simVecs`` of all ROIs.
		* If not generated yet, generates mesh (should never be the case!)
		* Initializes PDE with Neumann boundary conditions resulting in the problem:
		
		  .. math::
		     \partial_t c = D \nabla^2 c - k_1 c + k_2,
	
		  where :math:`k_1` is the degradation rate and :math:`k_2` the production rate.
		
		* Applies initial conditions defined in ``simulation.ICmode``.
		* Simulates FRAP experimment.
	
	Args: 
		simulation (pyfrp.subclasses.pyfrp_simulation.simulation): Simulation object.
	
	Keyword Args:
		signal (PyQt4.QtCore.pyqtSignal): PyQT signal to send progress to GUI.
		embCount (int): Counter of counter process if multiple datasets are analyzed. 
		debug (bool): Print final debugging messages and show debugging plots.
		showProgress (bool): Show simulation progress. 
		
	Returns: 
		pyfrp.subclasses.pyfrp_simulation.simulation: Updated simulation object.
	"""
	

	#Stepping and timescale
	timeStepDuration = simulation.tvecSim[1]-simulation.tvecSim[0]
	
	#Reset simulation vecs
	for r in simulation.embryo.ROIs:
		r.resetSimVec()
	
	#Empty list to put simulation values in
	vals=[]
	
	print "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
	print "Starting simulation"
	print "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
	
	startTimeTotal=time.clock()
	
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	#Mesh Generation
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	startTimeMesh=time.clock()
	
	if simulation.mesh.mesh==None:
		printWarning('No mesh has been generated yet!')
		a=raw_input('Do you want to generate a mesh now?[Y/N]')
		if a=='Y':
			simulation.mesh.genMesh()
			print "Mesh created in", time.clock()-startTimeMesh
		else:
			print 'Cannot run simulation without mesh, will abort.'
			return simulation
		
		timeMesh=time.clock()-startTimeMesh
		
	print "Mesh created after", time.clock()-startTimeTotal

	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	#Initialization of PDE
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	#Create solution variable
	
	phi = CellVariable(name = "solution variable",mesh = simulation.mesh.mesh,value = 0.) 
	

	#Apply initial conditions
	if simulation.ICmode==0:
		phi = applyROIBasedICs(phi,simulation)
		
	elif simulation.ICmode==1:
		phi = applyRadialICs(phi,simulation,debug=debug)
	
	elif simulation.ICmode==2:
		phi=applyImperfectICs(phi,simulation,simulation.embryo.geometry.getCenter(),100.,simulation.embryo.sliceHeightPx)
		
	elif simulation.ICmode==3:
		phi=applyInterpolatedICs(phi,simulation,debug=False)
		
	elif simulation.ICmode==4:
		phi=applyIdealICs(phi,simulation,bleachedROI=simulation.bleachedROI,valOut=simulation.valOut)
		
	#Remember ICs
	simulation.IC=np.asarray(phi.value).copy()
	
	#Defining Type of equation
	eq = TransientTerm() == DiffusionTerm(coeff=simulation.D)+simulation.prod-simulation.degr*phi

	#Defining BCs
	#Note: BCs are Neumann boundaries by default 
	
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	#Calculating initial concentrations 
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	for r in simulation.embryo.ROIs:
		r.getSimConc(phi,append=True)
	
	if simulation.saveSim:
		vals.append(np.asarray(phi.value).copy())
	
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	#Solving PDE
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	#Keeping track of time
	startTimeSim=time.clock()
	
	avgTime=0
	stepTime=0
	
	#Choose solver
	if simulation.solver=="LU":
		mySolver = LinearLUSolver(iterations=simulation.iterations, tolerance=simulation.tolerance)
	elif simulation.solver=="PCG":
		mySolver = LinearPCGSolver(tolerance=simulation.tolerance,iterations=simulation.iterations)

	for step in range(simulation.stepsSim-1):
		
		#Compute timestep duration 
		timeStepDuration=simulation.tvecSim[step+1]-simulation.tvecSim[step]
		
		#Solve PDE in this Step
		stepStart=time.clock()
		eq.solve(var=phi,dt=timeStepDuration,solver=mySolver)
		stepTime=stepTime+(time.clock()-stepStart)
				
		#Compute concentration
		avgStart=time.clock()
		
		for r in simulation.embryo.ROIs:
			r.getSimConc(phi,append=True)
		
		avgTime=avgTime+(time.clock()-avgStart)
		
		#Save simulation array if necessary
		if simulation.saveSim:
			vals.append(np.asarray(phi.value).copy())
		
		#Print Progress
		if showProgress:
			currPerc=int(100*step/float(simulation.stepsSim))
			
			if signal==None:
				sys.stdout.write("\r%d%%" %currPerc)  
				sys.stdout.flush()
			else:	
				if embCount==None:
					signal.emit(currPerc)
				else:
					signal.emit(currPerc,embCount)
			

	print "Step time: ", stepTime, " in %:", stepTime/(time.clock()-startTimeSim)*100
	print "Avg time: ", avgTime, " in %:", avgTime/(time.clock()-startTimeSim)*100
	print "Simulation done after", time.clock()-startTimeTotal
	
	#Save to simulation object only
	if simulation.saveSim:
		simulation.vals=list(vals)
	
	return simulation

def rerunReactDiff(simulation,signal=None,embCount=None,showProgress=True,debug=False):
	
	"""Reruns simulation by extracting values from ``simulation.vals``.
	
	Performs the following steps:
		
		* Resets ``simVecs`` of all ROIs.
		* Extracts values per ROI from ``simulation.vals``.
	
	.. note:: Only works if simulation has been run before with ``saveSim`` enabled.
	
	Args: 
		simulation (pyfrp.subclasses.pyfrp_simulation.simulation): Simulation object.
	
	Keyword Args:
		signal (PyQt4.QtCore.pyqtSignal): PyQT signal to send progress to GUI.
		embCount (int): Counter of counter process if multiple datasets are analyzed. 
		debug (bool): Print final debugging messages and show debugging plots.
		showProgress (bool): Show simulation progress. 
		
	Returns: 
		pyfrp.subclasses.pyfrp_simulation.simulation: Updated simulation object.
	"""
	
	#Check if can be rerun
	if len(simulation.vals)==0:
		printWarning("Values have not been saved for this simulation. Turn on saveSim to do that. Won't do anything for now.")
		return simulation
	
	#Reset simulation vecs
	for r in simulation.embryo.ROIs:
		r.resetSimVec()
	
	#Loop through vals
	for i,val in enumerate(simulation.vals):
		for r in simulation.embryo.ROIs:
			r.getSimConc(val,append=True)
		
		#Print Progress
		if showProgress:
			currPerc=int(100*i/float(len(simulation.vals)))
			
			if signal==None:
				sys.stdout.write("\r%d%%" %currPerc)  
				sys.stdout.flush()
			else:	
				if embCount==None:
					signal.emit(currPerc)
				else:
					signal.emit(currPerc,embCount)
		
	return simulation	
	

def applyROIBasedICs(phi,simulation):
	
	"""Applies ROI-based initial conditions.
	
	First sets concentration on all mesh nodes equal to `simulation.embryo.analysis.concRim`.
	Afterwards, mesh nodes get assigned the value of the first entry ``dataVec`` of 
	the ROI covering them. Note: If a mesh node is covered by two ROIs, will assign the value
	of the ROI that is last in embryo's ``ROIs`` list. See also 
	:py:func:`pyfrp.subclasses.pyfrp_simulation.setICMode`.
	
	Args:
		phi (fipy.CellVariable): PDE solution variable.
		simulation (pyfrp.subclasses.pyfrp_simulation.simulation): Simulation object.
	
	Returns:
		fipy.CellVariable: Updated solution variable.	
	
	"""
	
	phi.setValue(simulation.embryo.analysis.concRim)
	
	for r in simulation.embryo.ROIs:
		phi.value[r.meshIdx]=r.dataVec[0]
		
	
def applyIdealICs(phi,simulation,bleachedROI=None,valOut=None):
	
	"""Applies ideal initial conditions.
	
	That is, everything falling inside the bleached ROI in 
	x-y-direction will be set its initial dataVec value,
	everything else will be set equal to valOut.
	
	.. note:: The ``bleachedROI`` and valOut are often stored inside the simulation
	   object. If those two cannot be found, will try to find a ROI called *Bleached Square*
	   for the bleached ROI and set valOut to ``concRim``. If this again fails, will return
	   error.
	
	Args:
		phi (fipy.CellVariable): PDE solution variable.
		simulation (pyfrp.subclasses.pyfrp_simulation.simulation): Simulation object.
		
	Keyword Args:
		bleachedROI (pyfrp.subclasses.pyfrp_ROI.ROI): Bleached ROI.
		valOut (float): Value to be assigned outside of bleached ROI.
		
	Returns:
		fipy.CellVariable: Updated solution variable.	
	
	"""
	
	if bleachedROI==None:
		bleachedROI=simulation.embryo.getROIByName("Bleached Square")
		if bleachedROI==None:
			printError("No bleachedROI can be found in applyIdealICs.")
			return phi
		
	if valOut==None:
		if simulation.embryo.analysis.concRim!=None:
			valOut=simulation.embryo.analysis.concRim
		else:
			printError("No bleachedROI can be found in applyIdealICs.")
			return phi
	
	#Set all values in mesh to valOut
	phi.setValue(valOut)
	
	#Find indices of all nodes that lie inside bleachedROI in x-y-direction
	x,y,z=simulation.mesh.getCellCenters()
	ind=bleachedROI.checkXYInside(np.asarray(x),np.asarray(y))
	
	#Set these nodes equal to first dataVec entry
	phi.value[ind]=bleachedROI.dataVec[0]
	
	return phi
			
def applyRadialICs(phi,simulation,radSteps=15,debug=False):
	
	"""Applies radially averaged image data to solution variable as IC.
	
	.. note:: Will use ``embryo.geometry.center`` as center circle and the maximum 
	   distant pixel from the center as maximum radius.
	
	Args: 
		phi (fipy.CellVariable): PDE solution variable.
		simulation (pyfrp.subclasses.pyfrp_simulation.simulation): Simulation object.
	
	
		radSteps (int): Number of radial levels.
		debug (bool): Print debugging messages.
		
	Returns:
		fipy.CellVariable: Updated solution variable.
	
	"""
	
	#Adjust center so histogram works for 'quad'
	if 'quad' in simulation.embryo.analysis.process.keys():
		center=[0,0]
	else:
		center=simulation.ICimg,simulation.embryo.geometry.getCenter()
	
	#Compute radial histogram of IC image
	maxR=pyfrp_img_module.dist(center,[simulation.ICimg.shape[0],simulation.ICimg.shape[0]])
	bins,binsMid,histY,binY=pyfrp_img_module.radialImgHist(simulation.ICimg,nbins=radSteps,byMean=True,maxR=maxR)
	
	#Set center to actual center of geometry
	center=simulation.ICimg,simulation.embryo.geometry.getCenter()
	
	#Apply value of most outer bin to all nodes
	phi.setValue(binY[-1])
	
	#Loop through all bins and apply values from outside to inside
	binsY=binsY.reverse()
	bins=bins.reverse()
	for i in range(len(binY)):
		
		phi.setValue(binY[i], where=(x-center[0])**2+(y-center[1])**2 < bins[i]**2)
			
		if debug:
			print "Applied concentration", binY[i], " to all nodes with radius <", bins[i] 
	
	return phi
		

def applyInterpolatedICs(phi,simulation,matchWithMaster=True,debug=False,fixNeg=True,fillICWithConcRim=True):
	
	"""Interpolates initial conditions onto mesh.
	
	Uses a bivarariate spline interpolation (http://docs.scipy.org/doc/scipy-0.16.1/reference/generated/scipy.interpolate.RectBivariateSpline.html)
	to generate an interpolation function of the IC image. Then applies interpolated values to solution variable ``phi`` if mesh nodes are inside 
	image and masterROI. If not, will apply rim concentration.
	
	.. note:: If no rim concentration has been calculated (for example through running the data analysis)
	   applyInterpolatedICs will try to compute ``concRim`` by itself. For this it will take the mean concentration outside of bleached square but inside
	   ``masterROI``. 
	   
	.. note:: The bleached square used here is not defined as a ``ROI`` object here, but rather through the properties 
	   ``embryo.sideLengthBleachedPx`` and ``embryo.offsetBleachedPx``. This might change in future versions.
	
	Args: 
		phi (fipy.CellVariable): PDE solution variable.
		simulation (pyfrp.subclasses.pyfrp_simulation.simulation): Simulation object.
	
	Keyword Args:
		matchWithMaster (bool): Match interpolation indices with ``masterROI`` indices.
		debug (bool): Print debugging messages.
	
	Returns:
		fipy.CellVariable: Updated solution variable.
	"""
	
	#Get image resolution and center of geometry
	res=simulation.ICimg.shape[0]
	center=simulation.embryo.geometry.getCenter()
	
	#Define x/y coordinates of interpolation
	if 'quad' in simulation.embryo.analysis.process.keys():
		#Shift everything by center to fit with the mesh
		xInt = np.arange(center[0]+1, center[0]+res+1, 1)
		yInt = np.arange(center[1]+1, center[1]+res+1, 1)		
	else:
		xInt = np.arange(1, res+1, 1)
		yInt = np.arange(1, res+1, 1)
	
	#Getting cell centers
	x,y,z=simulation.mesh.getCellCenters()
	
	#Finding outer rim concentration
	if simulation.embryo.analysis.concRim==None:
		printWarning('concRim is not analyzed yet. Will use concentration outside of bleached region as approximation')
		
		#Grab offset and sidelength
		offset=simulation.embryo.offsetBleachedPx
		sidelength=simulation.embryo.sideLengthBleachedPx
		
		#Get indices outside of bleached square but inside masterROI
		indXSqu,indYSqu=pyfrp_idx_module.getSquareIdxImg(simulation.embryo.offsetBleachedPx,simulation.embryo.sideLengthBleachedPx,simulation.embryo.dataResPx)
		
		masterROI=simulation.embryo.getMasterROI()
		
		indX=pyfrp_misc_module.complValsSimple(masterROI.imgIdxX,indXSqu)
		indY=pyfrp_misc_module.complValsSimple(masterROI.imgIdxX,indYSqu)
		
		if 'quad' in simulation.embryo.analysis.process.keys():
			img=pyfrp_img_module.unflipQuad(np.flipud(simulation.ICimg))
		else:
			img=simulation.ICimg
		
		concRim=pyfrp_img_module.meanConc(img[indX,indY])
		
		print 'Approximate concRim = ', concRim
		
	else:	
		concRim=simulation.embryo.analysis.concRim
	
	if fillICWithConcRim:
		masterROI=simulation.embryo.getMasterROI()
		ICimg=concRim*np.ones(simulation.ICimg.shape)
		ICimg[masterROI.imgIdxX,masterROI.imgIdxY]=simulation.ICimg[masterROI.imgIdxX,masterROI.imgIdxY]	
	else:	
		ICimg=simulation.ICimg.copy()
	
	#Generate interpolation function
	f=interp.RectBivariateSpline(xInt, yInt, ICimg.T, bbox=[None, None, None, None], kx=3, ky=3, s=0)
	
	#Set all values of solution variable to concRim
	phi.setValue(concRim)
	
	#Get Offset of image and check which nodes are inside image
	if 'quad' in simulation.embryo.analysis.process.keys():
		offset=[simulation.embryo.dataResPx/2,simulation.embryo.dataResPx/2]
		ins=pyfrp_idx_module.checkInsideImg(x,y,simulation.embryo.dataResPx/2,offset=offset)
	else:
		offset=[0,0]
		ins=pyfrp_idx_module.checkInsideImg(x,y,simulation.embryo.dataResPx,offset=offset)
		
	#Convert into indices
	ind=np.arange(len(x))
	ind=ind[np.where(ins)[0]]

	"""NOTE:  I think we need to match here indices inside image with the one of master ROI, so we don't apply
	values outside of masterROI (generally just background) to nodes that lie INSIDE image, but OUTSIDE of masterROI.
	"""
	
	if matchWithMaster:
		
		masterROI=simulation.embryo.getMasterROI()
	
		xnew=np.asarray(x)[ind]
		ynew=np.asarray(y)[ind]
				
		ins=masterROI.checkXYInside(xnew,ynew)
		
		ind=np.asarray(ind)
		ind=ind[np.where(ins)[0]]
		ind=list(ind)
		
	#Apply interpolation
	try:
		phi.value[ind]=f.ev(x[ind],y[ind])
	except IndexError:
		if debug:
			printNote("Changed index array to nparray b/c of IndexError.")
		ind=np.array(ind)
		phi.value[ind]=f.ev(x[ind],y[ind])
	
	#Fix negative values if selected
	if fixNeg:
		phi=fixNegValues(phi)
	
	return phi
	
def fixNegValues(phi,minVal=None):
	
	"""Fixes negative values in solution variable. 
	
	Interpolation sometimes returns negative values if gradients are really steep. Will
	apply ``minVal`` to such nodes. 
	
	If ``minVal==None``, will take smallest non-negative value of solution value.
	
	
	
	"""
	
	if minVal==None:
		minVal=min(phi.value[np.where(phi.value>=0)[0]])
	
	phi.value[np.where(phi.value<0)[0]]=minVal
	
	return phi
	
def sigmoidBleachingFct(x,y,z,center,rJump,sliceHeight,maxVal=1.,maxMinValPerc=0.25,minMinValPerc=0.25,rate=0.1):
	
	r"""Generates sigmoid scaling function for imperfect bleaching at
	coordinates x/y/z.
	
	The idea behind the sigmoid function is:
	
		* Through scattering and other effects, the bleached window becomes blurry in larger depths, resulting
		  in a radial sigmoid scaling function around ``center``. 
		* Similarly, bleaching intensity increases with depth. Thus, a linear term controls the values close 
		  to ``center`` of the sigmoid function. Bleaching closer to the laser than the imaged height will 
		  be rendered stronger, while bleaching effects below will be decreased by *bumping up* the 
		  bleached window. However, only until some threshold is reached.
		  
	The sigmoid function is given by:	  
	
	.. math:: s(r,z) = v_{\mathrm{min}}(z)+(v_{\mathrm{max}}-v_{\mathrm{min}}(z))\frac{1}{1+\exp(-\rho(r-r_\mathrm{Jump}))},
	
	where :math:`\rho` is the sigmoid slope given by ``rate``, :math:`r_\mathrm{Jump}` is the radius from ``center``
	at which sigmoid function has its **jump**, given by ``rJump`` and :math:`r` the radius of coordinate ``[x,y]`` from
	``center``.
	
	:math:`v_{\mathrm{min}}(z)` is a linear function describing how strong the bleaching is dependent on the 
	depth :math:`z` given by
	
	.. math:: v_{\mathrm{min}}(z) = \frac{v_{\mathrm{max}} - v_{\mathrm{max. bleach}}}{h_s} z +  v_{\mathrm{max. bleach}},
	
	where :math:`v_{\mathrm{max}}` is the value of the sigmoid function far from ``center``, :math:`v_{\mathrm{max. bleach}}`
	is the strongest expected bleaching value (mostly at :math:`z=0`) and :math:`h_s` is the height of the imaging slice, given 
	by ``sliceHeight``. 
	The maximum rate of bleaching :math:`v_{\mathrm{max. bleach}}` is computed by:
	
	.. math:: v_{\mathrm{max. bleach}} = (1-p_{\mathrm{max. bleach}})v_{\mathrm{max}},
	
	where :math:`p_{\mathrm{max. bleach}}` is the percentage of maximum expected bleaching compared to the values in the imaging 
	height, given by ``maxMinValPerc``. That is, by how much has the laser power already decreased on its way from entry point of 
	the sample to the imaging height.
	
	For sample depths deeper than the imaging height, bleaching is expected to be decrease in intensity, thus the bleached area
	is getting **bumped up**. To avoid bumping the bleached area by too much, eventually even resulting in the bleached
	area having a higher concentration than the area outside, the sigmoid function has a cut-off: If values of :math:`s(r,z)` pass 
	
	.. math:: v_{\mathrm{min. bleach}} = (1+p_{\mathrm{min. bleach}})v_{\mathrm{max}},
	
	where :math:`p_{\mathrm{min. bleach}}` is the percentage of bleaching to cut-off, then we set
	
	.. math:: s(r,z) = v_{\mathrm{min. bleach}},
	
	ultimately resulting in a scaling function given by
	
	.. math:: s(r,z) = \left\{\begin{array}{cc} 
	   v_{\mathrm{min}}(z)+(v_{\mathrm{max}}-v_{\mathrm{min}}(z))\frac{1}{1+\exp(-\rho(r-r_\mathrm{Jump}))} & \mbox{ if } s(r,z) <= v_{\mathrm{min. bleach}} , \\
	   v_{\mathrm{min. bleach}} & \mbox{ else }
	   \end{array}
	   \right.
	   
	.. image:: ../imgs/pyfrp_sim_module/sigmoidFct.png
	
	Args:
		x (numpy.ndarray): x-coordinates.
		y (numpy.ndarray): y-coordinates.
		z (numpy.ndarray): z-coordinates.
		center (list): Center of bleaching. 
		rJump (float): Radius from center where sigmoid jump is expected.
		sliceHeight (float): Height at which dataset was recorded. 
		
	Keyword Args:	
		maxVal (float): Value of sigmoid function outside of bleached region.
		maxMinValPerc (float): Percentage of maximum bleaching intensity.
		minMinValPerc (float): Percentage of minimum bleaching intensity. 
		rate (float): Rate at which sigmoid increases.
			
	Returns:
		numpy.ndarray: 
		
	"""
	
	#Calculate linear equation describing how strong bleaching effect decreases as z increases
	maxMinVal=(1-maxMinValPerc)*maxVal
	m=(maxVal-maxMinVal)/sliceHeight
	b=maxMinVal
	minVal=m*z+b
	
	#Compute distance from center for each point
	r=np.sqrt((x-center[0])**2+(y-center[1])**2)
	
	#Compute sigmoid function
	sigm=minVal+(maxVal-minVal)/(1+np.exp(-rate*(r-rJump)))
	
	#Compute cut-off, so we do not amplifiy the bleached region
	minMinVal=(1+minMinValPerc)*maxVal
	sigm[np.where(sigm>minMinVal)]=minMinVal
	
	return sigm,r
	
def applyImperfectICs(phi,simulation,center,rJump,sliceHeight,maxVal=1.,maxMinValPerc=0.25,minMinValPerc=None,rate=0.1,matchWithMaster=True,debug=False):
	
	"""Mimic imperfect bleaching through cone approximation, return phi.
	
	 .. warning:: Not working in current version. Will be integrated in further versions again.
	
	"""
	
	phi = applyInterpolatedICs(phi,simulation,matchWithMaster=matchWithMaster,debug=debug)
	
	#Get cell coordinates
	x,y,z=simulation.mesh.getCellCenters()
	x=np.asarray(x)
	y=np.asarray(y)
	z=np.asarray(z)
	
	#Compute by how much 
	if minMinValPerc==None:
		r=np.sqrt((x-center[0])**2+(y-center[1])**2)
		inVal=np.mean(phi.value[np.where(r<rJump)])
		outVal=np.mean(phi.value[np.where(r>=rJump)])
		
		minMinValPerc=inVal/outVal
		

		
	#Compute sigmoid function
	sigm,r = sigmoidBleachingFct(x,y,z,center,rJump,sliceHeight,maxVal=1.,maxMinValPerc=maxMinValPerc,minMinValPerc=minMinValPerc,rate=rate)
	
	#Multiplicate solution variable with sigmoid function
	phi.value = phi.value * sigm

	return phi
	
	



		
	

